@@ -1,4 +1,5 @@
 import re
+import os
 import copy
 import time
 import queue
@@ -212,7 +213,7 @@ class Node:
 
     async def get_inps(self, queue):
         inps = []
-        for i in self.conf['inp']:
+        for i in self.conf.get('inp', []):
             if type(i) is str:
                 d = await queue[i].get()
                 inps.append(d)
@@ -857,6 +858,8 @@ class CodeNode(Node):
             inps_dict = {k:self._eval_format(v) for k,v in zip(self.conf['inp'], inps)}
             out = eval(self.conf['code'].format(**inps_dict))
         self.set_out(out, data, queue)
+        log.debug(f'{self.conf["out"]}: {out}')
+        self.execute_finish_callback(out)
 
     def current_mp_task(self, inps, data, queue, config=None):
         if 'code_entry' in self.conf:
@@ -869,6 +872,8 @@ class CodeNode(Node):
             inps_dict = {k:self._eval_format(v) for k,v in zip(self.conf['inp'], inps)}
             out = eval(self.conf['code'].format(**inps_dict))
         self.set_out(out, data, config=config)
+        log.debug(f'{self.conf["out"]}: {out}')
+        self.execute_finish_callback(out)
         for n in self.next: queue.put((n.name, config))
     
     def current_normal_task(self, inps, data, queue):
@@ -882,6 +887,8 @@ class CodeNode(Node):
             inps_dict = {k:self._eval_format(v) for k,v in zip(self.conf['inp'], inps)}
             out = eval(self.conf['code'].format(**inps_dict))
         self.set_out(out, data)
+        log.debug(f'{self.conf["out"]}: {out}')
+        self.execute_finish_callback(out)
 
 class WebNode(Node):
     mermaid_style = NodeColorStyle.WebNode
@@ -973,6 +980,8 @@ class ValueNode(Node):
         else:
             out = self.conf['value']
         self.set_out(out, data, config=config)
+        log.debug(f'{mode = }, {self.conf["out"]}: {out}')
+        self.execute_finish_callback(out)
         for n in self.next: queue.put((n.name, config))
 
     async def current_task(self, data, queue, dynamic_tasks):
@@ -994,6 +1003,8 @@ class ValueNode(Node):
         else:
             out = v
         self.set_out(out, data, queue)
+        log.debug(f'{mode = }, {self.conf["out"]}: {out}')
+        self.execute_finish_callback(out)
 
     def current_normal_task(self, inps, data, queue):
         mode = self.conf.get('mode', 'assign')
@@ -1013,6 +1024,8 @@ class ValueNode(Node):
         else:
             out = v
         self.set_out(out, data)
+        log.debug(f'{mode = }, {self.conf["out"]}: {out}')
+        self.execute_finish_callback(out)
 
 class DatabaseNode(Node):
     def aa(self):
@@ -1085,7 +1098,7 @@ class FileNode(Node):
 
         self.set_out(md, data)
         log.debug(f'[{self.name}] read: {info} -> {self.conf["out"]}')
-        
+
 
 class PipeTree:
     def __init__(self, llm_backend, rag_backend, prompt_manager, name=None, pipeconf:dict=None, pipefile=None, run_mode='async'):
@@ -1117,7 +1130,7 @@ class PipeTree:
         if type(pipefile) is str: pipefile = Path(pipefile)
         m = importlib.import_module(pipefile.stem)
         self.pipeconf = m.pipeline
-        if self.name is None: self.name = pipefile.stem
+        if self.name is None: self.name = pipefile.stem.removesuffix(os.getenv('PIPELINE_SUFFIX', '_pipeline'))
 
     def _check(self):
         node_names = set(self.node_manager.keys())
@@ -1184,7 +1197,7 @@ class PipeTree:
         if not self.start_nodes:
             log.error(f"Can't find start entry in pipes.")
             exit()
-        log.debug(f"'{self.name}' tree initialization successful, start nodes: {self.start_nodes}")
+        log.debug(f"'{self.name}' tree initialization successful, start nodes: {self.start_nodes}, required input: {self.required_inputs}")
 
     def export_conf(self):
         conf = copy.deepcopy(self.pipeconf)
@@ -1302,14 +1315,14 @@ class PipeTree:
         return mermaid
 
     def check_inp(self, inp):
-        need_inps = set()
-        for n in self.start_nodes: need_inps.update(n.mermaid_inps)
-        missing = need_inps - set(inp.keys())
-        assert not missing, f'missing input data: {list(missing)}'
+        if (missing := self.required_inputs - set(inp.keys())):
+            error_msg = f'missing input data: {list(missing)}'
+            log.error(f'[{self.name}]:\n{error_msg}')
+            return {'error_msg': error_msg}
 
     async def async_run(self, inp_data):
         self.reset()
-        self.check_inp(inp_data)
+        if (err := self.check_inp(inp_data)): return err
         data = copy.deepcopy(inp_data)
         dynamic_tasks = []
         queue = collections.defaultdict(asyncio.Queue)
@@ -1350,7 +1363,7 @@ class PipeTree:
 
     def mp_run(self, inp_data, core_num=4):
         self.reset()
-        self.check_inp(inp_data)
+        if (err := self.check_inp(inp_data)): return err
         task_queue = self.mp_manager.Queue()
         perf_queue = self.mp_manager.Queue()
         data       = self.mp_manager.dict()
@@ -1371,7 +1384,7 @@ class PipeTree:
 
     def normal_run(self, inp_data):
         self.reset()
-        self.check_inp(inp_data)
+        if (err := self.check_inp(inp_data)): return err
         data = copy.deepcopy(inp_data)
 
         try:
